'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.injectAsyncReducer = injectAsyncReducer;
exports.injectAsyncSagas = injectAsyncSagas;
exports.default = getAsyncInjectors;

var _invariant = require('invariant');

var _invariant2 = _interopRequireDefault(_invariant);

var _warning = require('warning');

var _warning2 = _interopRequireDefault(_warning);

var _ramda = require('ramda');

var _utils = require('./utils');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Injects asynchronously loaded reducer/s
 */
function injectAsyncReducer(store, createReducer, isValid) {
    return function injectReducer(name, asyncReducer) {
        if (!isValid) (0, _utils.checkStore)(store);

        (0, _invariant2.default)((0, _ramda.is)(String, name) && !(0, _ramda.isEmpty)(name) && (0, _ramda.is)(Object, asyncReducer) && (0, _ramda.is)(Function, createReducer), '(AsyncBundle/injectAsyncReducer): Expected `asyncReducer` to be a reducer function');

        if (Reflect.has(store.asyncReducers, name)) return;

        store.asyncReducers[name] = asyncReducer;
        store.replaceReducer(createReducer(store.asyncReducers));
    };
}

/**
 * Injects asynchronously loaded sagas
 */


/* eslint no-param-reassign: 0 */
function injectAsyncSagas(store, getAsyncSagas, isValid) {
    return function injectSagas(name, sagas) {
        if (!isValid) (0, _utils.checkStore)(store);

        (0, _invariant2.default)((0, _ramda.is)(String, name) && !(0, _ramda.isEmpty)(name) && (0, _ramda.is)(Array, sagas) && (0, _ramda.is)(Function, store.runSaga), '(AsyncBundle/injectAsyncSagas): Expected `sagas` to be an array of generator functions');

        (0, _warning2.default)(!(0, _ramda.isEmpty)(sagas), '(AsyncBundle/injectAsyncSagas): Received an empty `sagas` array');

        if (store.asyncSagas && Reflect.has(store.asyncSagas, name)) return;
        getAsyncSagas(name, sagas).map(store.runSaga);
    };
}

/**
 * Returns helper methods
 */
function getAsyncInjectors(store, createReducer) {
    if (!(0, _ramda.isNil)(store)) {
        (0, _utils.checkStore)(store);
    }
    return {
        injectAsyncReducer: injectAsyncReducer(store, createReducer, true),
        injectAsyncSagas: injectAsyncSagas(store, (0, _utils.getSagas)(store), true)
    };
}